//////////////////////////////////////////////////////////////////////
//
//  LLCommon.prg
//
//  Copyright: Copyright (C) combit GmbH
//
//  Content:
//  D: Allgemeine List & Label Routinen
//  US: Common List & Label routines
//////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "xbp.ch"
#include "common.ch"
#include "class.ch"
#include "dll.ch"

#include "cmbtll25.ch"
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
FUNCTION DefineData(hJob, cFieldname, xFieldValue, lAsField, lUnicode)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// D: Wird vom Programm aufgerufen, um die Daten entsprechend dem
//    neuen Datensatz zu definieren. In lAsField wird festgelegt,
//    ob die Daten als Felder oder als Variable an List & Label
//    uebergeben werden.
//    Daten werden aus dem aktuellen Arbeitsbereich ermittelt.
// US: Is called by the program to define the variables according
//    to the new record. lAsField distinguishes between field and
//    variable declaration to List & Label
//    Data is retrived from current workarea.

  LOCAL FldType, FldContent, DateBuffer, lExpr
  LOCAL nRet

  DEFAULT lUnicode TO .F.

  DO CASE

    CASE(ValType(xFieldValue) == "N")

        FldType := LL_NUMERIC
        FldContent := Str(xFieldValue)

    CASE(ValType(xFieldValue) == "D")

        FldType := LL_DATE
        DateBuffer := Replicate(chr(0), 255)

        // D: In Julianisches Datum konvertieren
        // US: Convert to Julian Date

        // D: Ausdruck erzeugen
        // US: Create expression
        lExpr := LlExprParse(hJob,"DateToJulian(DATE("+ Chr(34) + DToC(xFieldValue) + Chr(34) + "))", .F.)

        // D: Ausdruck auswerten
        // US: Evaluate expression
        LlExprEvaluate(hJob, lExpr, @DateBuffer, 255)

        // D: Ausdruck freigeben
        // US: Free expression
        LlExprFree(hJob, lExpr)
        FldContent := DateBuffer

    CASE(ValType(xFieldValue) == "L")

        FldType := LL_BOOLEAN
        FldContent := IIf(xFieldValue, "TRUE", "FALSE")

    CASE(ValType(xFieldValue) == "C")

        FldType := LL_TEXT

        IF(lUnicode)

          FldContent := xFieldValue

        ELSE

          FldContent := Trim(xFieldValue)

        ENDIF

    CASE(ValType(xFieldValue) == "M")

        FldType := LL_TEXT
        FldContent := xFieldValue

  ENDCASE

  // D: Daten an List & Label uebergeben
  // US: Pass data to List & Label
  IF(lAsField)

//    IF(lUnicode)
//
//       nRet := LlDefineFieldExtW(hJob, Str2Unicode(cFieldName) + Chr(0), FldContent, FldType, 0 )
//
//    ELSE
//
       nRet := LlDefineFieldExt(hJob, cFieldName, FldContent, FldType, 0 )

//    ENDIF

  ELSE

//     IF(lUnicode)
//
//       nRet := LlDefineVariableExtW(hJob, Str2Unicode(cFieldName), FldContent, FldType, 0 )
//
//     ELSE
//
       nRet := LlDefineVariableExt(hJob, cFieldName, FldContent, FldType, 0 )

//     ENDIF

  ENDIF

RETURN(nRet)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
FUNCTION PrintLLTableRow(hJob)

   // D: Tabellenzeile ausdrucken
   // US: Print table line
   LOCAL nRet := LlPrintFields(hJob)

   // D: Wenn Seitenumbruch, dann neue Kopfzeile drucken und alte Daten wiederholen
   // US: On page break print new header and repeat last data
   WHILE(nRet=LL_WRN_REPEAT_DATA)

     LlPrint(hJob)
     nRet := LlPrintFields(hJob)

   ENDDO

RETURN(nRet)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
FUNCTION PrintLLTableFooter(hJob)

  // D: Drucke Fusszeile der letzten Seite
  // US: Print footer of last page
  LOCAL nRet := LlPrintFieldsEnd(hJob)

  // D: Seitenumbruch fuer letzte Fusszeile, wenn noetig
  // US: Page break for last footer, if necessary
  WHILE(nRet=LL_WRN_REPEAT_DATA)

     nRet:=LlPrintFieldsEnd(hJob)

  ENDDO

RETURN(nRet)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// D: LL Fehlermeldung anzeigen
// US: Display LL error message
PROCEDURE LLErrorMessage(nErrCode)

  // Fehlermeldung
  LOCAL cErrMsg := Replicate(chr(0),400)

  IF(!Empty(nErrCode))

    LlGetErrortext(nErrCode, @cErrMsg, 390)
    ConfirmBox(NIL, "#"+AllTrim(Str(nErrCode))+" "+cErrMsg, "L&L error", XBPMB_OK,;
               XBPMB_QUESTION + XBPMB_APPMODAL + XBPMB_MOVEABLE)

  ENDIF

RETURN
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// D: Andere Systemfunktionen...
// US: Other system functions...
STATIC DLLFUNCTION GetTempPathA( buffsize, @buffer ) USING STDCALL FROM KERNEL32.DLL
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
FUNCTION MyGetTempPath()

  LOCAL nBuffSize := 261
  LOCAL sBuffer := Replicate(Chr(0), 261)

  GetTempPathA(nBuffsize, @sBuffer)

RETURN ALLTRIM(sBuffer)
//RETURN(RemAll(sBuffer, Chr(0)))
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
