//////////////////////////////////////////////////////////////////////
//
//  LULMENU.PRG
//
//  Copyright: Copyright (C) combit GmbH
//
//  Content:
//  D: List & Label Designer mit Echtdaten und Export via Men, basierend auf "LULMENU.PRG"
//  US: List & Label Design with real data and export, based on "LULMENU.PRG"
//  Changes and additions by Tom Liehr
//////////////////////////////////////////////////////////////////////

#include "Xbp.ch"
#include "dll.ch"
#include "common.ch"
#include "cmbtll25.ch"
#include "ot4xb.ch" // needed for structures in Xbase++. Free library from Pablo Botella @ xbwin.com
#include "bap.ch" // needed for callbacks in Xbase++. Get free from Alaska Software (ACSN)
#include "thread.ch"

#pragma library("bap.lib") // see above
#pragma library("ot4xb.lib") // see above

// D: Ab hier beginnen List & Label spezifische Design- und Druckroutinen
// US: List & Label specific print and design routines start here

DLLFUNCTION SetEvent(hEvent) USING STDCALL FROM KERNEL32.DLL // needed to inform L&L about the events

//-------------------------------------------------------------------
PROCEDURE InvoiceDesignRD(nRecNo)
//-------------------------------------------------------------------
Local hJob, nRet, sName := "Invoice"+Replicate(chr(0),255)
Local sSelectedNo
LOCAL nCount := 0
LOCAL nCallBack // adress of the callback function
LOCAL nWindowHandle := SetAppWindow():getHWND()


    // D: Fehler beim Laden der Library
    // US: Library loading error
    If !LL25ModuleInit()
      ConfirmBox(SetAppWindow(),"Error loading CMLL25.DLL !."+chr(13)+;
                                "OS error#"+Alltrim(Str(DosError()))+Chr(13)+;
                                DosErrorMessage(DosError())+chr(13)+;
                                Chr(13)+;
                                "Tip:"+chr(13)+;
                                "In later Xbase++ versions you can check with"+chr(13)+;
                                "'DosErrorMessage(DosError())' for error description.",;
                                "L&L DLL loading error",;
                                XBPMB_OK ,XBPMB_CRITICAL+XBPMB_APPMODAL+XBPMB_MOVEABLE )

      Return
    EndIf

    If Memvar->lDebugOutput
      // D: Debug-Modus aktivieren
      // US: Activate debug mode
      LlSetDebug(LL_DEBUG_CMBTLL)
    Endif

    // D: List & Label Job oeffnen und Handle holen
    // US: Open List & Label job, retrieve job handle
    hJob := LlJobOpen(-1)

    // D: Fehler - Druckprozess beenden...
    // US: Error - cancel printing process...
    If (hJob < 0)
       LLErrorMessage(hJob)
       Return
    Endif

    // D: Hier wird Ihr persoenlicher Lizenzinfostring eingetragen - siehe perslic.txt
    // US: Enter your licensing info string here - see perslic.txt
    LlSetOptionString (hJob, LL_OPTIONSTR_LICENSINGINFO, MyLicenseString()) // see end of code - set your license string there

    // D: Dateiauswahldialog oeffnen
    // US: Call file open dialog
    nRet := LlSelectFileDlgTitleEx(hJob, nWindowHandle, "Select File", LL_PROJECT_LIST, @sName, 255)


    ********************************************
    // D: Vorbereitung des Echtdatenpreviews. Abgesehen hiervon entspricht alles dem normalen Designeraufruf
    // US: Real data preview in designer - preparations. Besides this, same procedure as normal designer call

    LlSetOption(hJob,LL_OPTION_DESIGNERPREVIEWPARAMETER,nRecNo) // referenced record number as user parameter for the callback function

    ********************************************
    // D: Vorbereitung des Designerexports
    // US: Preparations for designer export
    LlSetOption(hJob,LL_OPTION_DESIGNEREXPORTPARAMETER,nRecNo)

    // Initialize callback function using BAP (if wanted - OT4XB provides a callback compiler, but his is much simpler:)
    nCallBack := BaCallBack("DesignerCallback", BA_CB_GENERIC3)

    // Hand callback function handle to L&L, so L&L can call the callback (callbackback <g>)
    LlSetNotificationCallbackExt(hJob,LL_NTFY_DESIGNERPRINTJOB,nCallBack)

    // D: Das war's schon
    // US: That's it
    ********************************************

    // D:  Tabellen ffnen, Daten suchen, ersten Datensatz publizieren
    // US: Open tables, find data, publish first record
    // D:  1. Auftrags-Kopfdaten
    // US: 1. Invoice header
    USE (GetDataPath()+"\INVOICE") NEW

    DbGoto(nRecNo)
    sSelectedNo := field->BILLNO

    // D: DLL-internen Felderpuffer loeschen
    // US: Clear DLL-internal field buffer
    LlDefineVariableStart(hJob)

    // D: Variablen aus Datenbank definieren
    // US: Define variables from database
    DefineData(hJob, .F.)

    CLOSE INVOICE

    // D:  2. Auftragspositionen
    // US: 2. Invoice items
    USE (GetDataPath()+"\ITEMS") NEW

    SET FILTER to field->billNo == sSelectedNo
    DbGoTop()
    // D: Anzahl Artikel auf Rechnung sSelectedNo ermitteln
    // US: Count how many items are on bill sSelectedNo
    nCount := 0
    Do While !Eof()
      nCount ++
      DbSkip(1)
    EndDo
    DbGoTop()

    // D: DLL-internen Felderpuffer loeschen
    // US: Clear DLL-internal field buffer
    LlDefineFieldStart(hJob)

    // D: Felder definieren
    // US: Define Fields
    DefineData(hJob, .T.)
    DefineArticleEANCode(hJob, field->ARTICLENO)

    CLOSE // Tables not needed anymore

    if nRet != LL_ERR_USER_ABORTED
      // D: Designer aufrufen
      // US: Start designer
      nRet := LlDefineLayout(hJob, nWindowHandle, "Designer", LL_PROJECT_LIST, sName)
    Endif

    // D: List & Label-Job beenden
    // US: Close List & Label job
    LlJobClose(hJob)
    LL25ModuleExit()

RETURN


********************************************
* US: Druckdatenroutine fr die Vorschau aus dem Designer
* Warning: Very low error handling
* D: Print data routine for preview from the designer
* Warnung: Kaum Fehlerbehandlung

PROCEDURE PrintPreviewData(sName,sOrgName,nWindowHandle,nRecNo,nPages,hEvent,lExport,cExpFormat,lWithoutDialog)

  LOCAL cTemporaryPath := MyGetTempPath()
  LOCAL nRet, hJob := LlJobOpen(-1) // open new job!
  LOCAL sSelectedNo,nCount,nAkt := 0
  LOCAL nPage :=1

  PrintRuns(.T.)
  SetEvent(hEvent)

  LlSetOptionString(hJob, LL_OPTIONSTR_ORIGINALPROJECTFILENAME,sOrgName) // associate the original (!) form name

  IF !lExport
     nRet := LlAssociatePreviewControl(hJob,nWindowHandle,1) // associate the window handle
  ELSE
     IF !Empty(cExpFormat)
        LlPrintSetOptionString(hJob, LL_PRNOPTSTR_EXPORT, cExpFormat)
     ENDIF
  ENDIF

  * publish data - as usual:

  USE (GetDataPath()+"\INVOICE") NEW

  DbGoto(nRecNo)
  sSelectedNo := field->BILLNO

  LlDefineVariableStart(hJob)

  DefineData(hJob, .F.)

  CLOSE

  USE (GetDataPath()+"\ITEMS") NEW

  SET FILTER to field->billNo == sSelectedNo
  DbGoTop()
  Do While !eof()
    nCount := RECCOUNT()
    DbSkip(1)
  EndDo
  DbGoTop()

  LlDefineFieldStart(hJob)

  DefineData(hJob, .T.)
  DefineArticleEANCode(hJob, field->ARTICLENO)

  IF !lExport // preview
     nRet := LlPrintStart(hJob, LL_PROJECT_LIST, sName, LL_PRINT_PREVIEW)

     LLSetOption(hJob,LL_OPTION_ESC_CLOSES_PREVIEW, 1)
     LlPreviewSetTempPath(hJob, cTemporaryPath)
  ELSE // export
     nRet := LlPrintWithBoxStart(hJob, LL_PROJECT_LIST, sName, LL_PRINT_EXPORT, LL_BOXTYPE_NORMALMETER, nWindowHandle, "Export")
     IF lWithoutDialog // format selected
        LlPrintSetOptionString(hJob,LL_PRNOPTSTR_EXPORT,cExpFormat)
     ELSE // no format selected
        nRet := LlPrintOptionsDialog(hJob, nWindowHandle, 'Export')
        IF nRet # 0 // error - nothing selected
          // some error handling would be needed here - close job, associate preview control (see end of function)
        ENDIF
     ENDIF
  ENDIF

  LlPrintSetOption(hJob,LL_PRNOPT_LASTPAGE,nPages)

  nRet := LlPrint(hJob)

  Do While !Eof() .AND. (nPages = 0 .or. nPage <= nPages) // nPages kann null sein

     nRet := DefineData(hJob, .T.)
     nRet := DefineArticleEANCode(hJob, field->ARTICLENO)

     nRet := PrintLLTableRow(hJob)

     If !Empty(nRet)
        LLErrorMessage(nRet)
        Exit
     Endif

     DbSkip(1)
     nPage := LlPrintGetCurrentPage(hJob)
     IF lExport
       nAkt ++
       LlPrintSetBoxText(hJob, 'Export', ( (100*nAkt)/nCount ))
     ENDIF
  EndDo

  nRet:= PrintLLTableFooter(hJob)

  nRet := LlPrintEnd(hJob, 0)

  If !Empty(nRet)
     LLErrorMessage(nRet)
  Endif

  CLOSE ALL

  IF !lExport
     LlAssociatePreviewControl(hJob,NIL,1) // release preview window
  ENDIF
  LlJobClose(hJob) // close 2nd print job
  FErase(sName) // remove temporary form file
  PrintRuns(.F.)
  SetEvent(hEvent)

RETURN

* Static function to check running print routines for each thread
FUNCTION PrintRuns(lSet) //
STATIC aPrintRuns := {} // array to set bool for print runs/is stopped for each thread - should be thread-safe then
LOCAL nFindThread := AScan(aPrintRuns,{|a|a[1]==ThreadID()})
IF nFindThread < 1
   aAdd(aPrintRuns,{ThreadID(),.F.})
   nFindThread := Len(aPrintRuns)
ENDIF
IF PCount()>0
   aPrintRuns[nFindThread,2] := lSet // maybe remove array element instead of setting it to .F., return .F. then
ENDIF
RETURN aPrintRuns[nFindThread,2]

*************************************************
* D: Die Callback-Funktion, die von L&L aufgerufen wird, um den Preview zu steuern
* US: The callback function for handling the preview

FUNCTION DesignerCallback(nNotification,nStructurePtr,xUserParam)
STATIC lThreadRuns := .F., oPrintThread
LOCAL oCbLocal,cProjectName,cOrgProjectName,nProjecthWnd,nRecNo,nPages,hEvent, cExpFormat,lWithoutDialog

IF nNotification == LL_NTFY_DESIGNERPRINTJOB

  * Create a local copy of the structure
  oCbLocal   := My_LlCallback():New():_link_(nStructurePtr,.F.) // get the structure where nStructurePtr points to - .F. means: not a copy
  oCbLocal:_lock_() // lock the structure to avoid interferences

  DO CASE
    // D: Vorschau, US: Preview
    CASE oCbLocal:_nFunction == LL_DESIGNERPRINTCALLBACK_PREVIEW_START
    // Init/retrieve values for the print thread
    cProjectName    := PeekStr(oCbLocal:_pszProjectName,,-1) // filename from structure pointer
    cOrgProjectName := PeekStr(oCbLocal:_pszOriginalProjectFileName,,-1) // filename from structure pointer
    nProjecthWnd    := oCbLocal:_hWnd // window handle
    nRecNo          := oCbLocal:_nUserParam // record number in INVOICE.DBF
    nPages          := oCbLocal:_nPages // number of pages to be printed
    hEvent          := oCbLocal:_hEvent // event no to be set by the printing routine
    // Start print thread
    oPrintThread := Thread():new()
    oPrintThread:start( {||PrintPreviewData(cProjectName,cOrgProjectName,nProjecthWnd,nRecNo,nPages,hEvent,.F.)} ) // last parameter: lExport := .F.
    lThreadRuns := .T.

    CASE oCbLocal:_nFunction == LL_DESIGNERPRINTCALLBACK_PREVIEW_ABORT
    lThreadRuns := PrintRuns(.F.)

    CASE oCbLocal:_nFunction == LL_DESIGNERPRINTCALLBACK_PREVIEW_FINALIZE
    cProjectName := PeekStr(oCbLocal:_pszProjectName,,-1)
    lThreadRuns := .F.
    FErase(cProjectName)

    CASE oCbLocal:_nFunction == LL_DESIGNERPRINTCALLBACK_PREVIEW_QUEST_JOBSTATE
    lThreadRuns := PrintRuns()

    // D: Ende der Echtdatenvorschau - US: End of preview

    // D/US: Export
    CASE oCbLocal:_nFunction == LL_DESIGNERPRINTCALLBACK_EXPORT_START
    // Init/retrieve values for the export thread
    cProjectName    := PeekStr(oCbLocal:_pszProjectName,,-1) // filename from structure pointer
    cOrgProjectName := PeekStr(oCbLocal:_pszOriginalProjectFileName,,-1) // filename from structure pointer
    nProjecthWnd    := oCbLocal:_hWnd // window handle
    nRecNo          := oCbLocal:_nUserParam // record number in INVOICE.DBF
    nPages          := oCbLocal:_nPages // number of pages to be printed
    hEvent          := oCbLocal:_hEvent // event no to be set by the printing routine
    cExpFormat      := PeekStr(oCbLocal:_pszExportFormat,,-1)
    lWithoutDialog  := oCbLocal:_bWithoutDialog
    // Start export thread
    oPrintThread := Thread():new()
    oPrintThread:start( {||PrintPreviewData(cProjectName,cOrgProjectName,nProjecthWnd,nRecNo,nPages,hEvent,.T.,cExpFormat,lWithoutDialog)} ) // last parameter: lExport := .T.
    lThreadRuns := .T.

    CASE oCbLocal:_nFunction == LL_DESIGNERPRINTCALLBACK_EXPORT_ABORT
    lThreadRuns := PrintRuns(.F.)

    CASE oCbLocal:_nFunction == LL_DESIGNERPRINTCALLBACK_EXPORT_FINALIZE
    cProjectName := PeekStr(oCbLocal:_pszProjectName,,-1)
    lThreadRuns := .F.
    FErase(cProjectName)

    CASE oCbLocal:_nFunction == LL_DESIGNERPRINTCALLBACK_EXPORT_QUEST_JOBSTATE
    lThreadRuns := PrintRuns()

    // D: Ende des Exports - US: End of export


  ENDCASE

  oCbLocal:_hEvent := IF(lThreadRuns,LL_DESIGNERPRINTTHREAD_STATE_RUNNING,LL_DESIGNERPRINTTHREAD_STATE_STOPPED) // set the event for L&L

  oCbLocal:_unlock_() // unlock the structure
  oCbLocal:_unlink_(.F.) // unlink the structure

ENDIF
RETURN IF(lThreadRuns,LL_DESIGNERPRINTTHREAD_STATE_RUNNING,LL_DESIGNERPRINTTHREAD_STATE_STOPPED)

FUNCTION MyLicenseString()
RETURN "" // your license string here!

// Create a structure using OT4XB
// this is precompiled to a function call, so it must stand outside the rest of the code
BEGIN STRUCTURE My_LlCallback
  MEMBER UINT         _nSize
  MEMBER POINTER      _nUserParam
  MEMBER POINTER32    _pszProjectName //
  MEMBER POINTER32    _pszOriginalProjectFileName
  MEMBER UINT         _nPages
  MEMBER UINT         _nFunction
  MEMBER HWND         _hWnd
  MEMBER HANDLE       _hEvent
  MEMBER POINTER32    _pszExportFormat
  MEMBER BOOL         _bWithoutDialog
END STRUCTURE
